import os
from pyresumize.interfaces import SkillBaseInterface
from pyresumize.utilities import Utilities


class SkillStandardEngine(SkillBaseInterface):
    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        self.nlp = nlp

    def process(self, extracted_text):
        """ """
        nlp_text = self.nlp(extracted_text)
        skills = []
        tokens = [token.text for token in nlp_text if not token.is_stop]
        # Iterate through the skills csvs and build info

        skills_input_folder = os.path.join(self.config_folder, "skills")
        utils = Utilities()
        skills = utils.generate_keywords_from_csv_files(skills_input_folder)
        skills = list(map(lambda x: str(x).lower(), skills))  # Normalising the Strings to Lower

        skillset = []

        # For words like JAVA
        for token in tokens:
            if token.lower() in skills:
                skillset.append(token)

        # for the Combined works such as Operating Systems
        for token in nlp_text.noun_chunks:
            token = token.text.lower().strip()
            if token in skills:
                skillset.append(token)
        # Lets mark it lower
        skillset = list(map(lambda x: str(x).lower(), skillset))
        # Remove duplicate skills, if any
        skillset = set(skillset)

        return skillset



from pdfminer.converter import TextConverter
from pdfminer.pdfinterp import PDFPageInterpreter
from pdfminer.pdfinterp import PDFResourceManager
from pdfminer.layout import LAParams
from pdfminer.pdfpage import PDFPage
from docx import Document
from docx.opc.exceptions import PackageNotFoundError
import io
import os.path


class AbstractFileProcessor:
    """The Abstract class provides an interface to extend the processing functonality
    The module supports PDF and DOCX, But in future if there is a new format comes in the functionality can be extended
    """

    def process(self, path):
        pass


class PDFFileProcessor(AbstractFileProcessor):
    """
    Using the PDF Miner Backend, process a Text file
    """

    def process(self, pdf_path):
        text = ""
        try:
            with open(pdf_path, "rb") as fh:
                # iterate over all pages of PDF document
                for page in PDFPage.get_pages(fh, caching=True, check_extractable=True):
                    resource_manager = PDFResourceManager()
                    fake_file_handle = io.StringIO()
                    converter = TextConverter(resource_manager, fake_file_handle, codec="utf-8", laparams=LAParams())
                    page_interpreter = PDFPageInterpreter(resource_manager, converter)
                    page_interpreter.process_page(page)
                    text += fake_file_handle.getvalue()
                    fake_file_handle.close()
        except Exception:
            return None
        return text


class DocXFileProcessor(AbstractFileProcessor):
    """
    Currently Uses the python-DocX Module to process the DocX Files
    """

    def process(self, filename):
        doc_text = None
        try:
            document = Document(filename)
        except ValueError as ve:
            doc_text = None
        except PackageNotFoundError:
            doc_text = None
        except Exception as e:
            doc_text = None
        else:
            doc_text = "\n\n".join(paragraph.text for paragraph in document.paragraphs)

        return doc_text


class TextProcessingFactory:
    """
    Factory Interface for text parsing
    """

    def __init__(self) -> None:
        self.processors = {}
        self.processors["docx"] = DocXFileProcessor()
        self.processors["pdf"] = PDFFileProcessor()

    def set_processor(self, customprocessor, extension):
        """
        We can define own processing engine and register it with factory via this method
        """
        self.processors[extension] = customprocessor

    def process(self, filename):
        """
        The worker
        """
        result = None
        extension = os.path.splitext(filename)[1][1:]
        if self.processors[extension]:
            result = self.processors[extension].process(filename)
        else:
            result = None
        return result



import re
from spacy.matcher import Matcher
from pyresumize.interfaces import PhoneBaseInterface, NameBaseInterface, EmailBaseInterface


class PhoneStandardEngine(PhoneBaseInterface):
    """
    Engine which uses a Regular expression to parse the phone number
    TODO: Currently it supports only international formats, Need improvement with better filterations
    """

    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        self.nlp = nlp

    def process(self, extracted_text):
        results = set(re.findall(r"[\+\(]?[1-9][0-9 .\-\(\)]{8,}[0-9]", extracted_text))
        phone = ",".join(results)
        return phone


class NameStandardEngine(NameBaseInterface):
    """
    Engine to get the Name of the candidate
    TODO : Only supports Two worded names , If there are three words , it is not supported
    """

    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        self.matcher = Matcher(nlp.vocab)
        self.nlp = nlp

    def process(self, extracted_text):
        nlp_text = self.nlp(extracted_text)
        # Identify the Names (Nouns)
        pattern = [{"POS": "PROPN"}, {"POS": "PROPN"}]
        self.matcher.add("NAME", [pattern])
        matches = self.matcher(nlp_text)

        for _match_id, start, end in matches:  # pylint: disable=unused-variable
            span = nlp_text[start:end]
            return span.text
        return ""


class EmailStandardEngine(EmailBaseInterface):
    """
    Process the email addresses from a Resume
    """

    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        self.nlp = nlp
        pass

    def process(self, extracted_text):
        result = ""
        # Regex for finding email
        email = re.findall(r"([^@|\s]+@[^@]+\.[^@|\s]+)", extracted_text)
        if email:
            try:
                result = email[0].split()[0].strip(";")
                if result is None:
                    result = ""
            except IndexError:
                return result
        return result



import os
import re
from nltk.corpus import stopwords
from pyresumize.interfaces import EducationBaseInterface
from pyresumize.utilities import Utilities


"""
Below Modules are standard implementations of the respective interfaces
The developer of the library can extend the functionality as in needed by pluggin in a custom module
"""
STOPWORDS = set(stopwords.words("english"))


class EducationStandardEngine(EducationBaseInterface):
    """
    Engine to process the education details from Resume
    """

    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        self.nlp = nlp
        pass

    def process(self, extracted_text):
        _text = self.nlp(extracted_text)
        _text = [sent.text.strip() for sent in _text.sents]

        degrees = {}
        education_input_folder = os.path.join(self.config_folder, "education")
        utils = Utilities()
        education_input = utils.generate_keywords_from_csv_files(education_input_folder)
        education_input = list(map(lambda x: str(x).lower(), education_input))
        # Lets search for Single words like BE
        for index, text in enumerate(_text):
            for tex in text.split():
                # Replace all special symbols
                tex = re.sub(r"[?|$|.|!|,]", r"", tex)
                if tex.lower() in education_input and tex not in STOPWORDS:
                    if len(_text) > index + 1:
                        degrees[tex] = text + _text[index + 1]
        # Now Lets search Longer word like Bachelor of Technology
        # for the Combined works such as Operating Systems
        # TODO , Use Noun Chunks
        # Improve this please
        education = {}
        education_map = []
        for key in degrees.keys():
            year = re.search(
                re.compile(r"(((20)(\d{2})))"), degrees[key]
            )  # Assumes that the year is somewhere in 2000s
            if year:
                education_map.append((key, "".join(year[0])))
            else:
                education_map.append(key)
        education["entry"] = education_map
        education["universities"] = self.__find_universities(extracted_text)
        # Later can be added with university
        return education

    def __find_universities(self, resume_text):
        """ """
        nlp_text = self.nlp(resume_text)

        # tokens = [token.text for token in nlp_text if not token.is_stop]
        universities_input_folder = os.path.join(self.config_folder, "universities")
        utils = Utilities()
        universities = utils.generate_keywords_from_csv_files(universities_input_folder)
        universities = list(map(lambda x: str(x).lower(), universities))  # Normalising the Strings to Lower

        candidate_universities = []

        for token in nlp_text.noun_chunks:
            token = token.text.lower().strip()  # for the Combined works such as Operating Systems
            if token in universities:
                candidate_universities.append(token)

        return candidate_universities



import spacy
import nltk
import logging
import json

from pyresumize.utilities import Utilities
from pyresumize.basic_details_module import (
    NameStandardEngine,
    PhoneStandardEngine,
    EmailStandardEngine,
)
from pyresumize.education_module import EducationStandardEngine
from pyresumize.skills_module import SkillStandardEngine
from pyresumize.employment_module import EmployerStandardEngine
from pyresumize.text_processors import TextProcessingFactory

config_folder = "data"

logging.basicConfig(
    level=logging.WARNING, filename="pyresumize.log", filemode="w", format="%(name)s - %(levelname)s - %(message)s"
)

# TODO , See if this need to be done in another place, performance improvements.
nltk.download("stopwords")
# TODO End


class Candidate:
    """The Person Class who owns the given resume
    The Extracted data is encapsulated in this class
    """

    personal_details = {}
    skills = []
    education = {}
    employers = []


class ResumeEngine:
    """The GodFather Class with the API interface"""

    def __init__(self) -> None:
        # move this outside , loading a set takes time
        self.nlp = spacy.load("en_core_web_lg")  # Must have for names
        self.candidate = Candidate()
        self.name_engine = NameStandardEngine(self.nlp, config_folder)
        self.skills_engine = SkillStandardEngine(self.nlp, config_folder)
        self.phone_engine = PhoneStandardEngine(self.nlp, config_folder)
        self.email_engine = EmailStandardEngine(self.nlp, config_folder)
        self.education_engine = EducationStandardEngine(self.nlp, config_folder)
        self.employer_engine = EmployerStandardEngine(self.nlp, config_folder)
        self.text_factory = TextProcessingFactory()

    def set_skills_engine(self, engine):
        self.skills_engine = engine

    def set_name_engine(self, engine):
        self.name_engine = engine

    def set_phone_engine(self, engine):
        self.phone_engine = engine

    def set_email_engine(self, engine):
        self.email_engine = engine

    def set_education_engine(self, engine):
        self.education_engine = engine

    def set_employer_engine(self, engine):
        self.employer_engine = engine

    def set_custom_keywords_folder(self, folder_name):
        global config_folder
        config_folder = folder_name

    def add_custom_text_processor(self, processor, file_extension):
        self.text_factory.set_processor(processor, file_extension)

    def __generate_json(self, file_path):
        """
        Generates the respective json format of the resume
        """
        json_data = {}
        json_data["file_name"] = file_path
        json_data["basic_details"] = self.candidate.personal_details
        json_data["skills"] = ",".join(self.candidate.skills)
        json_data["education"] = self.candidate.education
        json_data["employers"] = ",".join(self.candidate.employers)
        return json.dumps(json_data)

    def process_resume(self, file_path):
        """
        The Worker API !
        """
        resume_data = self.text_factory.process(file_path)
        if resume_data == None:
            util = Utilities()
            return util.error_handler("File %s is not supported" % (file_path))

        self.candidate.skills = self.skills_engine.process(resume_data)
        self.candidate.personal_details["name"] = self.name_engine.process(resume_data)
        self.candidate.personal_details["phone"] = self.phone_engine.process(resume_data)
        self.candidate.personal_details["email"] = self.email_engine.process(resume_data)
        self.candidate.education = self.education_engine.process(resume_data)
        self.candidate.employers = self.employer_engine.process(resume_data)

        return self.__generate_json(file_path)



import os
import re
import spacy
from glob import glob
from pyresumize.interfaces import EmployerBaseInterface
from pyresumize.utilities import Utilities
import logging


class EmployerStandardEngine(EmployerBaseInterface):
    def __init__(self, nlp, config_folder) -> None:
        super().__init__(config_folder)
        # We overrides the default nlp given
        self.nlp = spacy.load("en_core_web_sm", exclude=["entity_ruler"])
        self.__generate_employers()

    def __generate_employers(self):
        employ_input_folder = os.path.join(self.config_folder, "employers")
        utils = Utilities()
        employers = utils.generate_keywords_from_csv_files(employ_input_folder)
        employers = list(map(lambda x: str(x).lower(), employers))  # Normalising the Strings to Lower
        print("found %d employers " % len(employers))
        patterns = []
        ruler = self.nlp.add_pipe("entity_ruler")
        for employer in employers:
            entry = {}
            entry["label"] = "EMPLOYER"
            entry["pattern"] = str(employer)
            patterns.append(entry)
        ruler.add_patterns(patterns)
        self.nlp.to_disk("employers")
        self.nlp_entity = spacy.load("employers")
        self.nlp.remove_pipe("entity_ruler")
        self.nlp.add_pipe("entity_ruler", source=self.nlp_entity)

    def process(self, employment_text):
        """Process the Custom Entity EMPLOYER"""

        doc = self.nlp(employment_text.lower())
        candidate_employment = []
        for ent in doc.ents:
            if ent.label_ == "EMPLOYER":
                candidate_employment.append(ent.text.lower())
                # print(ent.text)
                # print(ent.label_)
        # Removes Duplicate
        candidate_employment = set(candidate_employment)
        return candidate_employment
